
# q-expansion of newform 5376.2.c.s, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5376
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2815, [1, 0]], [5125, [-1, 0]], [1793, [1, 0]], [4609, [1, 0]]]
aps_data = [[0, 0], [0, 1], [0, 2], [1, 0], [0, 4], [0, -6], [-2, 0], [0, 4], [4, 0], [0, -2], [8, 0], [0, 10], [2, 0], [0, -8], [0, 0], [0, 10], [0, 12], [0, 10], [0, -8], [-12, 0], [-2, 0], [0, 0], [0, 12], [-6, 0], [2, 0], [0, 10], [8, 0], [0, 12], [0, -2], [-14, 0], [8, 0], [0, -20], [-18, 0], [0, -20], [0, -6], [16, 0], [0, 2], [0, 16], [0, 0], [0, 6], [0, -20], [0, -2], [12, 0], [-14, 0], [0, -6], [-16, 0], [0, -16], [-8, 0], [0, 20], [0, -18], [14, 0], [12, 0], [-30, 0], [0, 4], [-18, 0], [12, 0], [0, -2], [-32, 0], [0, 18], [30, 0], [0, 12], [0, -6], [0, 4], [-8, 0], [14, 0], [0, -2], [0, 8], [-14, 0], [0, 28], [0, 18], [6, 0], [12, 0], [-24, 0], [0, 34], [0, 8], [-8, 0], [0, -38], [0, 34], [10, 0], [22, 0], [0, 12], [0, 34], [36, 0], [-6, 0], [-8, 0], [0, -28], [18, 0], [6, 0], [0, -26], [16, 0], [0, 36], [0, 0], [32, 0], [0, 36], [0, -16], [0, 0], [0, -18], [-38, 0], [0, -28], [0, -26], [0, -32], [0, 30], [0, -20], [-18, 0], [0, -16], [2, 0], [0, -12], [-42, 0], [36, 0], [-10, 0], [8, 0], [0, 10], [-18, 0], [0, 20], [0, 0], [26, 0], [0, 28], [16, 0], [0, 14], [0, 44], [0, -18], [50, 0], [0, 18], [0, -28], [0, 12], [0, 30], [0, -38], [8, 0], [40, 0], [0, -14], [0, 32], [28, 0], [16, 0], [0, 26], [-30, 0], [26, 0], [0, 42], [0, -4], [0, -42], [-10, 0], [0, 4], [0, 10], [-32, 0], [0, -4], [0, -14], [16, 0], [0, -42], [18, 0], [0, -12], [52, 0], [0, 14], [-42, 0], [0, 16], [16, 0], [0, -48], [-12, 0], [-8, 0], [22, 0], [-10, 0], [0, 6], [0, 44], [54, 0], [-8, 0], [0, -28], [-22, 0], [-24, 0], [8, 0], [0, 62], [2, 0], [0, -30], [0, -36], [0, -30], [20, 0], [38, 0], [32, 0], [2, 0], [0, -40], [0, -6], [-32, 0], [0, -6], [8, 0], [0, -12], [0, -38], [2, 0], [-20, 0], [0, -30], [0, -10], [0, -4], [6, 0], [-8, 0], [-22, 0], [0, 12], [0, 0], [0, -42], [0, -44], [-14, 0], [-14, 0], [0, -2], [-2, 0], [56, 0], [0, 30], [0, 0], [0, -10], [10, 0], [0, -12], [0, -66], [64, 0], [0, -36], [6, 0], [0, 36], [-62, 0], [0, 2], [-16, 0], [0, -12], [40, 0], [46, 0], [56, 0], [46, 0], [-20, 0], [0, -34], [0, -6], [-32, 0], [66, 0], [-16, 0], [0, 28], [0, 26], [-6, 0], [12, 0], [32, 0], [0, 60], [0, 14], [0, 28], [-24, 0], [54, 0], [0, -4], [0, 0], [34, 0], [0, 26], [0, -12], [-16, 0], [0, -28], [0, -60], [72, 0], [0, -26], [-18, 0], [-48, 0], [-48, 0], [0, 52], [0, 16], [36, 0], [0, 22], [38, 0], [-28, 0], [62, 0], [0, -18], [0, -12], [0, 26], [0, -28], [0, -6], [14, 0], [-40, 0], [0, -12], [0, -10], [0, -22], [78, 0], [0, -28], [0, -34], [-30, 0], [0, 8], [0, -38], [0, -14], [0, -72], [-34, 0], [80, 0], [26, 0], [64, 0], [0, -12], [0, -26], [6, 0], [0, -28], [-48, 0], [80, 0], [-56, 0], [0, -2], [0, 52], [-44, 0], [34, 0], [0, -6], [-40, 0], [-2, 0], [0, -66], [0, 60], [-34, 0], [0, 60], [0, -26], [0, -74], [24, 0], [0, 58], [0, 20], [0, 36], [46, 0], [0, -18], [32, 0], [0, 60], [0, 32], [-14, 0], [0, -36], [0, -30], [76, 0], [0, -46], [0, 0], [0, -54], [-54, 0], [0, -40], [-4, 0], [30, 0], [0, -12], [20, 0], [-6, 0], [10, 0], [0, -36], [-26, 0], [0, -50], [24, 0], [54, 0], [-14, 0], [0, 16], [0, 28], [-12, 0], [0, -38], [0, -10], [0, 62], [40, 0], [0, -84], [0, -56], [0, 28], [0, 14], [30, 0], [-66, 0], [-8, 0], [0, -78], [-66, 0], [0, 42], [-8, 0], [0, 30], [0, 60], [0, -34], [0, 88], [32, 0], [0, -54], [0, 60], [-58, 0], [0, -50], [0, 0], [0, -46], [2, 0], [-24, 0], [0, -44], [-30, 0], [44, 0], [0, -14], [-70, 0], [4, 0], [0, 60], [0, 12], [-90, 0], [0, 46], [56, 0], [70, 0], [0, -84], [0, 20], [60, 0], [0, -22], [-80, 0], [0, 38], [0, 76], [-36, 0], [-14, 0], [-82, 0], [-42, 0], [0, 70], [-74, 0], [72, 0], [18, 0], [0, 68], [-48, 0], [-56, 0], [0, 22], [0, -40], [-48, 0], [18, 0], [0, -62], [0, 12], [0, -68], [60, 0], [-26, 0], [0, 0], [42, 0], [0, 8], [0, 42], [0, 26], [18, 0], [64, 0], [-78, 0], [0, 74], [0, 0], [0, 38], [-14, 0], [0, 20], [0, -20], [34, 0], [0, 10], [0, 44], [0, -80], [38, 0], [0, -66], [-12, 0], [-8, 0], [62, 0], [24, 0], [0, -2], [0, 78], [-36, 0], [0, 12], [-82, 0], [0, 94], [0, 4], [-18, 0], [0, 12], [16, 0]]
